local allowed = nil
local warned = {}
local lastAllowedIncomingPlayer = nil
local disabled = nil

local blockMessage = "ShutUp: Your message has been blocked, since you are not in my friends list or my guild."

local function ShouldShow(name)
	return allowed[name] or ShutUpDB[name]
end

function ShutUp_Disable() disabled = true end
function ShutUp_Enable() disabled = nil end

local orig = ChatFrame_MessageEventHandler
ChatFrame_MessageEventHandler = function(event, ...)
	if not allowed or (type(arg6) == "string" and arg6:find("GM")) or disabled then return orig(event, ...) end
	if type(ShutUpDB) == "table" and type(arg1) == "string" and type(arg2) == "string" then
		local player = arg2
		local text = arg1
		if event == "CHAT_MSG_WHISPER" then
			if not ShouldShow(player) then
				if not warned[player] then
					SendChatMessage(blockMessage, "WHISPER", nil, player)
					warned[player] = true
				end
				if lastAllowedIncomingPlayer then
					ChatEdit_SetLastTellTarget(lastAllowedIncomingPlayer)
				end
				return
			else
				lastAllowedIncomingPlayer = player
			end
		elseif event == "CHAT_MSG_WHISPER_INFORM" then
			if text:find(blockMessage) then
				return
			end
			if not allowed[player] then
				ShutUpDB[player] = true
			end
		end
	end
	return orig(event, ...)
end

local function InsertFriends()
	if not allowed then allowed = {} end
	-- Find all friends
	local numFriends = GetNumFriends()
	for i = 1, numFriends do
		local name = GetFriendInfo(i)
		if name then
			allowed[name] = true
		end
	end
end

local isUpdatingGuild = nil
local function InsertGuildMembers()
	if isUpdatingGuild then return end
	if not allowed then allowed = {} end
	-- Find all guild members.
	isUpdatingGuild = true
	local offline = GetGuildRosterShowOffline()
	local selection = GetGuildRosterSelection()
	SetGuildRosterShowOffline(true)
	SetGuildRosterSelection(0)
	GetGuildRosterInfo(0)
	local numGuildMembers = GetNumGuildMembers()
	for i = 1, numGuildMembers do
		local name = GetGuildRosterInfo(i)
		if name then
			allowed[name] = true
		end
	end
	SetGuildRosterShowOffline(offline)
	SetGuildRosterSelection(selection)
	isUpdatingGuild = nil
end


local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function()
	if event == "PLAYER_LOGIN" then
		if type(ShutUpDB) ~= "table" then ShutUpDB = {} end
	elseif event == "PARTY_MEMBERS_CHANGED" then
		if not allowed then allowed = {} end
		for i = 1, GetNumPartyMembers() do
			allowed[UnitName("party"..i)] = true
		end
	elseif event == "RAID_ROSTER_UPDATE" then
		if not allowed then allowed = {} end
		for i = 1, GetNumRaidMembers() do
			allowed[UnitName("raid"..i)] = true
		end
	elseif event == "FRIENDLIST_UPDATE" then
		InsertFriends()
	elseif event == "GUILD_ROSTER_UPDATE" then
		InsertGuildMembers()
	end
end)
do
	local totalTime = 0
	local function update()
		totalTime = totalTime + arg1
		if totalTime > 30 then
			-- 30 seconds after login, lets try to get guild information now.
			if IsInGuild() or GetNumFriends() > 0 then
				local numG = GetNumGuildMembers()
				local numF = GetNumFriends()
				if (type(numG) == "number" and numG > 0) or (type(numF) == "number" and numF > 0) then
					InsertGuildMembers()
					InsertFriends()
				else
					totalTime = 0
					return
				end
			end
			frame:SetScript("OnUpdate", nil)
		end
	end
	frame:SetScript("OnUpdate", update)
end

frame:RegisterEvent("GUILD_ROSTER_UPDATE")
frame:RegisterEvent("FRIENDLIST_UPDATE")
frame:RegisterEvent("PLAYER_LOGIN")
frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
frame:RegisterEvent("RAID_ROSTER_UPDATE")

GuildRoster()

